@@ -61,51 +61,36 @@ public class Xpp3ReaderGenerator
 
     private static final String LOCATION_VAR = "_location";
 
-    private ModelClass locationTracker;
-
     private String locationField;
 
-    private ModelClass sourceTracker;
-
     private String trackingArgs;
 
-    private boolean requiresDomSupport;
-
-    protected boolean isLocationTracking()
-    {
-        return false;
-    }
-
-    public void generate( Model model, Properties parameters )
+    @Override
+    protected void initialize( Model model, Properties parameters )
         throws ModelloException
     {
-        initialize( model, parameters );
+        super.initialize( model, parameters );
 
-        locationTracker = sourceTracker = null;
         trackingArgs = locationField = "";
-        requiresDomSupport = false;
 
         if ( isLocationTracking() )
         {
-            locationTracker = model.getLocationTracker( getGeneratedVersion() );
-            if ( locationTracker == null )
-            {
-                throw new ModelloException( "No model class has been marked as location tracker"
-                                                + " via the attribute locationTracker=\"locations\""
-                                                + ", cannot generate extended reader." );
-            }
-
             locationField =
                 ( (ModelClassMetadata) locationTracker.getMetadata( ModelClassMetadata.ID ) ).getLocationTracker();
 
-            sourceTracker = model.getSourceTracker( getGeneratedVersion() );
-
             if ( sourceTracker != null )
             {
                 trackingArgs += ", " + SOURCE_PARAM;
             }
         }
 
+    }
+
+    public void generate( Model model, Properties parameters )
+        throws ModelloException
+    {
+        initialize( model, parameters );
+
         try
         {
             generateXpp3Reader();
@@ -453,6 +438,11 @@ public class Xpp3ReaderGenerator
         if ( requiresDomSupport )
         {
             writeBuildDomMethod( jClass );
+
+            if ( isLocationTracking() )
+            {
+                writeBuildDomLocationTrackingMethod( jClass );
+            }
         }
 
         // ----------------------------------------------------------------------
@@ -1080,9 +1070,14 @@ public class Xpp3ReaderGenerator
         }
         else if ( "DOM".equals( type ) )
         {
-            sc.add( objectName + "." + setterName + "( " + keyCapture + ( domAsXpp3
-                ? "org.codehaus.plexus.util.xml.Xpp3DomBuilder.build"
-                : "buildDom" ) + "( parser, " + xmlFieldMetadata.isTrim() + " ) );" );
+            String locationBuilderParam = "";
+            if ( isLocationTracking() && domAsXpp3 )
+            {
+                locationBuilderParam = ", new Xpp3DomBuilderInputLocationBuilder( " + LOCATION_VAR + " )";
+            }
+            sc.add( objectName + "." + setterName + "( " + keyCapture
+                + ( domAsXpp3 ? "org.codehaus.plexus.util.xml.Xpp3DomBuilder.build" : "buildDom" )
+                + "( parser, " + xmlFieldMetadata.isTrim() + locationBuilderParam + " ) );" );
 
             requiresDomSupport = true;
         }
@@ -1104,6 +1099,7 @@ public class Xpp3ReaderGenerator
             // no need, Xpp3DomBuilder provided by plexus-utils
             return;
         }
+
         jClass.addField( new JField( new JClass( "org.w3c.dom.Document" ), "_doc_" ) );
         JMethod method = new JMethod( "initDoc", null, null );
         method.getModifiers().makePrivate();
@@ -1163,7 +1159,7 @@ public class Xpp3ReaderGenerator
         sc.add( "}" );
         sc.add( "else" );
         sc.add( "{" );
-        sc.addIndented( "values.push( new StringBuffer() );" );
+        sc.addIndented( "values.push( new StringBuilder() );" );
         sc.add( "}" );
 
         sc.add( "int attributesSize = parser.getAttributeCount();" );
@@ -1184,7 +1180,7 @@ public class Xpp3ReaderGenerator
         sc.add( "else if ( eventType == XmlPullParser.TEXT )" );
         sc.add( "{" );
         sc.indent();
-        sc.add( "StringBuffer valueBuffer = (StringBuffer) values.peek();" );
+        sc.add( "StringBuilder valueBuffer = (StringBuilder) values.peek();" );
 
         sc.add( "String text = parser.getText();" );
 
@@ -1228,6 +1224,34 @@ public class Xpp3ReaderGenerator
         jClass.addMethod( method );
     }
 
+    private void writeBuildDomLocationTrackingMethod( JClass jClass )
+    {
+        if ( !domAsXpp3 )
+        {
+            // no need, input location tracking available only for Xpp3
+            return;
+        }
+
+        JClass builderClass = jClass.createInnerClass( "Xpp3DomBuilderInputLocationBuilder" );
+        builderClass.getModifiers().makePrivate();
+        builderClass.getModifiers().setStatic( true );
+        builderClass.addInterface( "org.codehaus.plexus.util.xml.Xpp3DomBuilder.InputLocationBuilder" );
+
+        JField field = new JField( new JType( locationTracker.getName() ), "rootLocation" );
+        field.getModifiers().setFinal( true );
+        builderClass.addField( field );
+
+        JConstructor constructor = new JConstructor( builderClass );
+        constructor.addParameter( new JParameter( new JType( locationTracker.getName() ), "rootLocation" ) );
+        constructor.getSourceCode().add( "this.rootLocation = rootLocation;" );
+        builderClass.addConstructor( constructor );
+
+        JMethod method = new JMethod( "toInputLocation", new JType( "Object" ), null );
+        method.addParameter( new JParameter( new JType( "XmlPullParser" ), "parser" ) );
+        method.getSourceCode().add( "return " + buildNewLocation( "rootLocation.getSource()" ) + ";" );
+        builderClass.addMethod( method );
+    }
+
     private void writeHelpers( JClass jClass )
     {
         jClass.addMethod(getTrimmedValueMethod());
@@ -1624,11 +1648,14 @@ public class Xpp3ReaderGenerator
             return;
         }
 
-        String constr = "new " + locationTracker.getName() + "( parser.getLineNumber(), parser.getColumnNumber()";
-        constr += ( sourceTracker != null ) ? ", " + SOURCE_PARAM : "";
-        constr += " )";
+        sc.add( ( ( trackerVariable != null ) ? trackerVariable : LOCATION_VAR ) + " = "
+            + buildNewLocation( SOURCE_PARAM ) + ";" );
+    }
 
-        sc.add( ( ( trackerVariable != null ) ? trackerVariable : LOCATION_VAR ) + " = " + constr + ";" );
+    private String buildNewLocation( String source )
+    {
+        return "new " + locationTracker.getName() + "( parser.getLineNumber(), parser.getColumnNumber()"
+            + ( ( sourceTracker != null ) ? ", " + source : "" ) + " )";
     }
 
     private void writeSetLocation( String key, String objectName, String trackerVariable, JSourceCode sc )
