@@ -23,6 +23,7 @@ package org.codehaus.modello.plugin.xpp3;
  */
 
 import org.codehaus.modello.ModelloException;
+import org.codehaus.modello.ModelloParameterConstants;
 import org.codehaus.modello.model.Model;
 import org.codehaus.modello.model.ModelAssociation;
 import org.codehaus.modello.model.ModelClass;
@@ -52,15 +53,39 @@ import java.util.Properties;
 public class Xpp3WriterGenerator
     extends AbstractXpp3Generator
 {
-    private boolean requiresDomSupport;
+    private String extendedClassnameSuffix;
+
+    protected void prepareLocationTracking( JClass jClass )
+    {
+        // NO OP
+    }
+
+    protected void writeLocationTracking( JSourceCode sc, String name, String key )
+    {
+        if ( isLocationTracking() )
+        {
+            sc.add( "writeLocationTracking( " + name + ", " + key + ", serializer );" );
+        }
+    }
+
+    @Override
+    protected void initialize( Model model, Properties parameters )
+        throws ModelloException
+    {
+        super.initialize( model, parameters );
+
+        extendedClassnameSuffix = "Ex";
+        if ( isLocationTracking() )
+        {
+            extendedClassnameSuffix = parameters.getProperty( ModelloParameterConstants.EXTENDED_CLASSNAME_SUFFIX );
+        }
+    }
 
     public void generate( Model model, Properties parameters )
         throws ModelloException
     {
         initialize( model, parameters );
 
-        requiresDomSupport = false;
-
         try
         {
             generateXpp3Writer();
@@ -79,7 +104,7 @@ public class Xpp3WriterGenerator
         String packageName = objectModel.getDefaultPackageName( isPackageWithVersion(), getGeneratedVersion() )
             + ".io.xpp3";
 
-        String marshallerName = getFileName( "Xpp3Writer" );
+        String marshallerName = getFileName( "Xpp3Writer" + ( isLocationTracking() ? extendedClassnameSuffix : "" ) );
 
         JSourceWriter sourceWriter = newJSourceWriter( packageName, marshallerName );
 
@@ -184,6 +209,11 @@ public class Xpp3WriterGenerator
 
         writeAllClasses( objectModel, jClass );
 
+        if ( isLocationTracking() )
+        {
+            prepareLocationTracking( jClass );
+        }
+
         if ( requiresDomSupport )
         {
             createWriteDomMethod( jClass );
@@ -353,6 +383,8 @@ public class Xpp3WriterGenerator
 
                     if ( ModelDefault.LIST.equals( type ) || ModelDefault.SET.equals( type ) )
                     {
+                        boolean isList = ModelDefault.LIST.equals( type );
+
                         sc.add( getValueChecker( type, value, association ) );
 
                         sc.add( "{" );
@@ -363,6 +395,15 @@ public class Xpp3WriterGenerator
                             sc.add( "serializer.startTag( NAMESPACE, " + "\"" + fieldTagName + "\" );" );
                         }
 
+                        if ( isLocationTracking() && !isClassInModel( association.getTo(), modelClass.getModel() ) )
+                        {
+                            sc.add( locationTracker.getName() + " location = " + uncapClassName + ".getLocation( \"" + fieldTagName + "\" );" );
+                            if ( isList )
+                            {
+                                sc.add( "int n = 0;" );
+                            }
+                        }
+
                         sc.add( "for ( Iterator iter = " + value + ".iterator(); iter.hasNext(); )" );
 
                         sc.add( "{" );
@@ -376,12 +417,14 @@ public class Xpp3WriterGenerator
                         }
                         else
                         {
-                            sc.add( toType + " " + singular( uncapitalise( field.getName() ) ) + " = (" + toType
-                                + ") iter.next();" );
+                            String variable = singular( uncapitalise( field.getName() ) );
 
-                            sc.add( "serializer.startTag( NAMESPACE, " + "\"" + valuesTagName + "\" ).text( "
-                                + singular( uncapitalise( field.getName() ) ) + " ).endTag( NAMESPACE, " + "\""
-                                + valuesTagName + "\" );" );
+                            sc.add( toType + " " + variable + " = (" + toType + ") iter.next();" );
+
+                            sc.add( "serializer.startTag( NAMESPACE, \"" + valuesTagName + "\" ).text( " + variable
+                                + " ).endTag( NAMESPACE, \"" + valuesTagName + "\" );" );
+
+                            writeLocationTracking( sc, "location", isList ? "Integer.valueOf( n++ )" : variable );
                         }
 
                         sc.unindent();
@@ -389,7 +432,7 @@ public class Xpp3WriterGenerator
 
                         if ( wrappedItems )
                         {
-                            sc.add( "serializer.endTag( NAMESPACE, " + "\"" + fieldTagName + "\" );" );
+                            sc.add( "serializer.endTag( NAMESPACE, \"" + fieldTagName + "\" );" );
                         }
 
                         sc.unindent();
@@ -406,7 +449,12 @@ public class Xpp3WriterGenerator
 
                         if ( wrappedItems )
                         {
-                            sc.add( "serializer.startTag( NAMESPACE, " + "\"" + fieldTagName + "\" );" );
+                            sc.add( "serializer.startTag( NAMESPACE, \"" + fieldTagName + "\" );" );
+                        }
+
+                        if ( isLocationTracking() )
+                        {
+                            sc.add( locationTracker.getName() + " location = " + uncapClassName + ".getLocation( \"" + fieldTagName + "\" );" );
                         }
 
                         sc.add( "for ( Iterator iter = " + value + ".keySet().iterator(); iter.hasNext(); )" );
@@ -430,15 +478,17 @@ public class Xpp3WriterGenerator
                         else
                         {
                             sc.add(
-                                "serializer.startTag( NAMESPACE, \"\" + key + \"\" ).text( value ).endTag( NAMESPACE, \"\" + key + \"\" );" );
+                                "serializer.startTag( NAMESPACE, key ).text( value ).endTag( NAMESPACE, key );" );
                         }
 
+                        writeLocationTracking( sc, "location", "key" );
+
                         sc.unindent();
                         sc.add( "}" );
 
                         if ( wrappedItems )
                         {
-                            sc.add( "serializer.endTag( NAMESPACE, " + "\"" + fieldTagName + "\" );" );
+                            sc.add( "serializer.endTag( NAMESPACE, \"" + fieldTagName + "\" );" );
                         }
 
                         sc.unindent();
@@ -456,8 +506,15 @@ public class Xpp3WriterGenerator
                     if ( domAsXpp3 )
                     {
                         jClass.addImport( "org.codehaus.plexus.util.xml.Xpp3Dom" );
-    
-                        sc.addIndented( "((Xpp3Dom) " + value + ").writeToSerializer( NAMESPACE, serializer );" );
+
+                        if ( isLocationTracking() )
+                        {
+                            sc.addIndented( "writeXpp3DomToSerializer( (Xpp3Dom) " + value + ", serializer );" );
+                        }
+                        else
+                        {
+                            sc.addIndented( "((Xpp3Dom) " + value + ").writeToSerializer( NAMESPACE, serializer );" );
+                        }
                     }
                     else
                     {
@@ -471,6 +528,9 @@ public class Xpp3WriterGenerator
                     sc.addIndented( "serializer.startTag( NAMESPACE, " + "\"" + fieldTagName + "\" ).text( "
                         + getValue( field.getType(), value, xmlFieldMetadata ) + " ).endTag( NAMESPACE, " + "\""
                         + fieldTagName + "\" );" );
+                    sc.indent();
+                    writeLocationTracking( sc, uncapClassName, '"' + fieldTagName + '"' );
+                    sc.unindent();
                 }
                 sc.add( "}" );
             }
